# -*- coding: utf-8 -*-
from setuptools import setup

modules = \
['pipen_args']
install_requires = \
['pipen', 'pyparam']

setup_kwargs = {
    'name': 'pipen-args',
    'version': '0.0.0',
    'description': 'Provide command-line argument parser for pipen.',
    'long_description': "# pipen-args\n\nCommand line argument parser for [pipen][1]\n\n## Usage\n```python\nfrom pipen import Proc, Pipen\nfrom pipen_args import params\n\nclass Process(Proc):\n    input_keys = 'a'\n    input = range(10)\n    script = 'echo {{in.a}}'\n\nPipen(starts=Process).run()\n```\n\n```\n> python example.py --help\n11-04 11:46:08 I /main                        _____________________________________   __\n               I /main                        ___  __ \\___  _/__  __ \\__  ____/__  | / /\n               I /main                        __  /_/ /__  / __  /_/ /_  __/  __   |/ /\n               I /main                        _  ____/__/ /  _  ____/_  /___  _  /|  /\n               I /main                        /_/     /___/  /_/     /_____/  /_/ |_/\n               I /main\n               I /main                                     version: 0.0.1\n               I /main\n               I /main    ┏━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━ pipeline-0 ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┓\n               I /main    ┃ Undescribed.                                                                 ┃\n               I /main    ┗━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┛\n               I /main    Enabled plugins: ['verbose', 'main-0.0.1', 'args-0.0.0']\n               I /main    Loaded processes: 1\n\nDESCRIPTION:\n  Undescribed.\n\nUSAGE:\n  example.py [OPTIONS]\n\nOPTIONAL OPTIONS:\n  --profile <STR>                 - The default profile from the configuration\n                                    to run the pipeline. Default: default\n                                    This profile will be used unless a profile\n                                    is specified in the process or in the .run\n                                    method of pipen.\n  --loglevel <STR>                - The logging level for the main logger, only\n                                    takes effect after pipeline is initialized.\n                                    Default: debug\n  --cache [BOOL]                  - Whether enable caching for processes.\n                                    Default: True\n  --dirsig <INT>                  - The depth to check the Last Modification\n                                    Time of a directory. Default: 1\n                                    Since modify the content of a directory\n                                    won't change its LMT.\n  --error_strategy <CHOICE>       - How we should deal with job errors.\n                                    Default: ignore\n                                     - ignore: Let other jobs keep running.\n                                    But the process is still failing when done.\n                                     - halt: Halt the pipeline, other running\n                                    jobs will be killed.\n                                     - retry: Retry this job on the scheduler\n                                    system.\n  --num_retries <INT>             - How many times to retry the job when failed.\n                                    Default: 3\n  --forks <INT>                   - How many jobs to run simultaneously on the\n                                    scheduler system. Default: 1\n  --submission_batch <INT>        - How many jobs to submit simultaneously to\n                                    the scheduler system. Default: 8\n  --workdir <PATH>                - The workdir for the pipeline.\n                                    Default: ./.pipen\n  --envs <JSON>                   - The env variables for template rendering.\n                                    Default: {}\n                                    Will update to the default one.\n  --scheduler <STR>               - The default scheduler Default: local\n  --scheduler_opts <JSON>         - The default scheduler options. Will update\n                                    to the default one. Default: {}\n  --plugins <LIST>                - A list of plugins to only enabled or\n                                    disabled for this pipeline. Default: []\n                                    To disable plugins, use no:<plugin_name>\n  --plugin_opts <JSON>            - Plugin options. Will update to the default\n                                    one. Default: {}\n  --outdir <PATH>                 - The output directory for the pipeline.\n                                    Default: ./pipeline-0-output\n  -h, --help                      - Print help information for this command\n```\n\n[1]: https://github.com/pwwang/pipen\n",
    'author': 'pwwang',
    'author_email': 'pwwang@pwwang.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/pwwang/pipen-args',
    'py_modules': modules,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



"""Command line argument parser for pipen"""
from pyparam import Params
from pipen.plugin import plugin
from pipen.utils import logger

__version__ = '0.0.0'
__all__ = ['params']

name = 'args'

params = Params(help_on_void=False)

@plugin.impl
def on_init(pipen):
    """Setup and parse the arguments when the pipeline is initialized."""
    config = pipen.config
    params.add_param(
        'profile',
        default='default',
        desc=('The default profile from the configuration to run the pipeline.',
              'This profile will be used unless a profile is specified in the '
              'process or in the .run method of pipen.')
    )
    params.add_param(
        'loglevel',
        default=config.loglevel,
        desc=('The logging level for the main logger, only takes effect '
              'after pipeline is initialized.'),
        callback=lambda val: val and val.upper()
    )
    params.add_param(
        'cache',
        default=config.cache,
        type=bool,
        desc=('Whether enable caching for processes.')
    )
    params.add_param(
        'dirsig',
        default=config.dirsig,
        type=int,
        desc=('The depth to check the Last Modification Time of a directory.',
              'Since modify the content of a directory won\'t change its LMT.')
    )
    params.add_param(
        'error_strategy',
        default=config.error_strategy,
        type='choice',
        choices=['ignore', 'halt', 'retry'],
        desc=('How we should deal with job errors.',
              ' - `ignore`: Let other jobs keep running. '
              'But the process is still failing when done.',
              ' - `halt`: Halt the pipeline, other running jobs will be '
              'killed.',
              ' - `retry`: Retry this job on the scheduler system.')
    )
    params.add_param(
        'num_retries',
        default=config.num_retries,
        type=int,
        desc='How many times to retry the job when failed.'
    )
    params.add_param(
        'forks',
        default=config.forks,
        type=int,
        desc='How many jobs to run simultaneously on the scheduler system.'
    )
    params.add_param(
        'submission_batch',
        default=config.submission_batch,
        type=int,
        desc='How many jobs to submit simultaneously to the scheduler system.'
    )
    params.add_param(
        'workdir',
        default=config.workdir,
        type='path',
        desc='The workdir for the pipeline.'
    )
    params.add_param(
        'envs',
        default=config.envs,
        type='json',
        desc=('The env variables for template rendering. ',
              'Will update to the default one.')
    )
    params.add_param(
        'scheduler',
        default=config.scheduler,
        type=str,
        desc='The default scheduler'
    )
    params.add_param(
        'scheduler_opts',
        default=config.scheduler_opts,
        type='json',
        desc='The default scheduler options. Will update to the default one.'
    )
    params.add_param(
        'plugins',
        default=config.plugins,
        type=list,
        desc=(
            'A list of plugins to only enabled or disabled for this pipeline.',
            'To disable plugins, use `no:<plugin_name>`'
        )
    )
    params.add_param(
        'plugin_opts',
        default=config.plugin_opts,
        type='json',
        desc='Plugin options. Will update to the default one.'
    )
    params.add_param(
        'outdir',
        default=pipen.outdir,
        type='path',
        desc='The output directory for the pipeline.'
    )

    params.desc = [pipen.desc]
    parsed = params.parse()
    pipen.profile = parsed.profile
    pipen.outdir = parsed.outdir
    if parsed.loglevel != config.loglevel:
        logger.setLevel(parsed.loglevel)

    args = {'default': {}}
    if parsed.cache != config.cache:
        args['default']['cache'] = parsed.cache
    if parsed.dirsig != config.dirsig:
        args['default']['dirsig'] = parsed.dirsig
    if parsed.error_strategy != config.error_strategy:
        args['default']['error_strategy'] = parsed.error_strategy
    if parsed.num_retries != config.num_retries:
        args['default']['num_retries'] = parsed.num_retries
    if parsed.forks != config.forks:
        args['default']['forks'] = parsed.forks
    if parsed.submission_batch != config.submission_batch:
        args['default']['submission_batch'] = parsed.submission_batch
    if parsed.workdir != config.workdir:
        args['default']['workdir'] = parsed.workdir
    if parsed.envs != config.envs:
        envs = config.envs.copy()
        envs.update(parsed.envs)
        args['default']['envs'] = envs
    if parsed.scheduler != config.scheduler:
        args['default']['scheduler'] = parsed.scheduler
    if parsed.scheduler_opts != config.scheduler_opts:
        scheduler_opts = config.scheduler_opts.copy()
        scheduler_opts.update(parsed.scheduler_opts)
        args['default']['scheduler_opts'] = scheduler_opts
    if parsed.plugins != config.plugins:
        args['default']['plugins'] = parsed.plugins
    if parsed.plugin_opts != config.plugin_opts:
        plugin_opts = config.plugin_opts.copy()
        plugin_opts.update(parsed.plugin_opts)
        args['default']['plugin_opts'] = plugin_opts

    pipen.config._load(args)

plugin.register(__name__)
